@@ -6,8 +6,8 @@
  * See COPYING in top-level directory.
  */
 
-#include <private/autogen/config.h>
-#include <hwloc.h>
+#include "private/autogen/config.h"
+#include "hwloc.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -57,7 +57,7 @@ void usage(const char *name, FILE *where)
   fprintf (where, "  --thissystem          Assume that the input topology provides the topology\n"
 		  "                        for the system on which we are running\n");
   fprintf (where, "  --pid <pid>           Detect topology as seen by process <pid>\n");
-  fprintf (where, "  --whole-system        Do not consider administration limitations\n");
+  fprintf (where, "  --disallowed          Include objects disallowed by administrative limitations\n");
   fprintf (where, "  -l --logical          Use logical object indexes for input (default)\n");
   fprintf (where, "  -p --physical         Use physical object indexes for input\n");
   fprintf (where, "Miscellaneous options:\n");
@@ -131,6 +131,7 @@ hwloc_info_show_obj(hwloc_topology_t topology, hwloc_obj_t obj, const char *type
   case HWLOC_OBJ_L1ICACHE:
   case HWLOC_OBJ_L2ICACHE:
   case HWLOC_OBJ_L3ICACHE:
+  case HWLOC_OBJ_MEMCACHE:
     printf("%s attr cache depth = %u\n", prefix, obj->attr->cache.depth);
     switch (obj->attr->cache.type) {
     case HWLOC_OBJ_CACHE_UNIFIED: printf("%s attr cache type = Unified\n", prefix); break;
@@ -360,8 +361,10 @@ main (int argc, char *argv[])
   hwloc_utils_check_api_version(callname);
 
   /* enable verbose backends */
-  putenv((char *) "HWLOC_XML_VERBOSE=1");
-  putenv((char *) "HWLOC_SYNTHETIC_VERBOSE=1");
+  if (!getenv("HWLOC_XML_VERBOSE"))
+    putenv((char *) "HWLOC_XML_VERBOSE=1");
+  if (!getenv("HWLOC_SYNTHETIC_VERBOSE"))
+    putenv((char *) "HWLOC_SYNTHETIC_VERBOSE=1");
 
   err = hwloc_topology_init (&topology);
   if (err)
@@ -455,9 +458,10 @@ main (int argc, char *argv[])
           hwloc_topology_set_all_types_filter(topology, filter);
 	else if (allio)
           hwloc_topology_set_io_types_filter(topology, filter);
-	else if (allcaches)
+	else if (allcaches) {
 	  hwloc_topology_set_cache_types_filter(topology, filter);
-	else if (allicaches)
+	  hwloc_topology_set_type_filter(topology, HWLOC_OBJ_MEMCACHE, filter);
+	} else if (allicaches)
 	  hwloc_topology_set_icache_types_filter(topology, filter);
         else
           hwloc_topology_set_type_filter(topology, type, filter);
@@ -465,8 +469,8 @@ main (int argc, char *argv[])
       }
       else if (!strcmp (argv[0], "--no-icaches")) {
 	hwloc_topology_set_icache_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_NONE);
-      } else if (!strcmp (argv[0], "--whole-system"))
-	flags |= HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM;
+      } else if (!strcmp (argv[0], "--disallowed") || !strcmp (argv[0], "--whole-system"))
+	flags |= HWLOC_TOPOLOGY_FLAG_INCLUDE_DISALLOWED;
       else if (!strcmp (argv[0], "--no-io")) {
 	hwloc_topology_set_io_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_NONE);
       } else if (!strcmp (argv[0], "--no-bridges")) {
@@ -607,8 +611,10 @@ main (int argc, char *argv[])
     const struct hwloc_topology_support *support = hwloc_topology_get_support(topology);
 #define DO(x,y) printf(#x ":" #y " = %u\n", (unsigned char) support->x->y);
     DO(discovery, pu);
+    DO(discovery, disallowed_pu);
     DO(discovery, numa);
     DO(discovery, numa_memory);
+    DO(discovery, disallowed_numa);
 
     DO(cpubind, set_thisproc_cpubind);
     DO(cpubind, get_thisproc_cpubind);
