@@ -6,8 +6,8 @@
  * See COPYING in top-level directory.
  */
 
-#include <private/autogen/config.h>
-#include <hwloc.h>
+#include "private/autogen/config.h"
+#include "hwloc.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -28,9 +28,8 @@ static void
 output_console_obj (struct lstopo_output *loutput, hwloc_obj_t l, int collapse)
 {
   FILE *output = loutput->file;
-  int logical = loutput->logical;
+  enum lstopo_index_type_e index_type = loutput->index_type;
   int verbose_mode = loutput->verbose_mode;
-  unsigned idx = logical ? l->logical_index : l->os_index;
   char pidxstr[16];
   char lidxstr[32];
   char busidstr[32];
@@ -43,7 +42,7 @@ output_console_obj (struct lstopo_output *loutput, hwloc_obj_t l, int collapse)
     snprintf(lidxstr, sizeof(lidxstr), "L#%u", l->logical_index);
   }
   if (l->type == HWLOC_OBJ_PCI_DEVICE)
-    lstopo_busid_snprintf(busidstr, sizeof(busidstr), l, collapse, loutput->need_pci_domain);
+    lstopo_busid_snprintf(loutput, busidstr, sizeof(busidstr), l, collapse, loutput->need_pci_domain);
 
   if (loutput->show_cpuset < 2) {
     char type[64], *attr, phys[32] = "";
@@ -53,13 +52,20 @@ output_console_obj (struct lstopo_output *loutput, hwloc_obj_t l, int collapse)
       fprintf(output, "%s(%s)", type, l->subtype);
     else
       fprintf(output, "%s", type);
-    if (l->depth != 0 && idx != HWLOC_UNKNOWN_INDEX
-	&& (verbose_mode >= 2 || (hwloc_obj_type_is_normal(l->type) || hwloc_obj_type_is_memory(l->type))))
-      fprintf(output, " %s", logical ? lidxstr : pidxstr);
+    if (l->depth != 0 && (verbose_mode >= 2 || (hwloc_obj_type_is_normal(l->type) || hwloc_obj_type_is_memory(l->type)))) {
+      if (index_type != LSTOPO_INDEX_TYPE_PHYSICAL)
+	/* print logical index in logical and default case */
+	fprintf(output, " %s", lidxstr);
+      else if (index_type == LSTOPO_INDEX_TYPE_PHYSICAL && l->os_index != HWLOC_UNKNOWN_INDEX)
+	/* print physical index in physical case */
+	fprintf(output, " %s", pidxstr);
+    }
     if (l->name && (l->type == HWLOC_OBJ_MISC || l->type == HWLOC_OBJ_GROUP))
       fprintf(output, " %s", l->name);
-    if (logical && l->os_index != HWLOC_UNKNOWN_INDEX /* if logical, still print os_index in some cases */
+    if (index_type == LSTOPO_INDEX_TYPE_DEFAULT
+	&& l->os_index != HWLOC_UNKNOWN_INDEX
 	&& (verbose_mode >= 2 || l->type == HWLOC_OBJ_PU || l->type == HWLOC_OBJ_NUMANODE))
+      /* print physical index too if default index */
       snprintf(phys, sizeof(phys), "%s", pidxstr);
     if (l->type == HWLOC_OBJ_PCI_DEVICE && verbose_mode <= 1)
       fprintf(output, " %s (%s)",
@@ -111,16 +117,16 @@ output_console_obj (struct lstopo_output *loutput, hwloc_obj_t l, int collapse)
     free(cpusetstr);
   }
 
-  /* annotate if the PU/NUMA is forbidden/binding */
+  /* annotate if the PU/NUMA is disallowed/binding */
   if (verbose_mode >= 2) {
     if (l->type == HWLOC_OBJ_PU) {
-      if (lstopo_pu_forbidden(loutput, l))
-	fprintf(output, " (forbidden)");
+      if (lstopo_pu_disallowed(loutput, l))
+	fprintf(output, " (disallowed)");
       else if (lstopo_pu_binding(loutput, l))
 	fprintf(output, " (binding)");
     } else if (l->type == HWLOC_OBJ_NUMANODE) {
-      if (lstopo_numa_forbidden(loutput, l))
-	fprintf(output, " (forbidden)");
+      if (lstopo_numa_disallowed(loutput, l))
+	fprintf(output, " (disallowed)");
       else if (lstopo_numa_binding(loutput, l))
 	fprintf(output, " (binding)");
     }
@@ -135,7 +141,7 @@ output_topology (struct lstopo_output *loutput, hwloc_obj_t l, hwloc_obj_t paren
   int verbose_mode = loutput->verbose_mode;
   hwloc_obj_t child;
   int group_identical = (verbose_mode <= 1) && !loutput->show_cpuset;
-  int collapse = ((struct lstopo_obj_userdata *) l->userdata)->pci_collapsed;
+  int collapse = loutput->pci_collapse_enabled ? ((struct lstopo_obj_userdata *) l->userdata)->pci_collapsed : 0;
 
   if (l->type == HWLOC_OBJ_PCI_DEVICE && collapse == -1)
     return;
@@ -205,7 +211,7 @@ output_only (struct lstopo_output *loutput, hwloc_obj_t l)
 static void output_distances(struct lstopo_output *loutput)
 {
   hwloc_topology_t topology = loutput->topology;
-  int logical = loutput->logical;
+  enum lstopo_index_type_e index_type = loutput->index_type;
   FILE *output = loutput->file;
   struct hwloc_distances_s **dist;
   unsigned nr = 0, j;
@@ -219,13 +225,20 @@ static void output_distances(struct lstopo_output *loutput)
   if (!err) {
     for(j=0; j<nr; j++) {
       const char *kindmeans = (dist[j]->kind & HWLOC_DISTANCES_KIND_MEANS_LATENCY) ? "latency" : (dist[j]->kind & HWLOC_DISTANCES_KIND_MEANS_BANDWIDTH) ? "bandwidth" : "distance";
-      fprintf(output, "Relative %s matrix (kind %lu) between %u %ss (depth %d) by %s indexes:\n",
-	      kindmeans, dist[j]->kind,
-	      dist[j]->nbobjs,
-	      hwloc_obj_type_string(dist[j]->objs[0]->type),
-	      dist[j]->objs[0]->depth,
-	      logical ? "logical" : "physical");
-      hwloc_utils_print_distance_matrix(output, dist[j]->nbobjs, dist[j]->objs, dist[j]->values, logical);
+      if (dist[j]->kind & HWLOC_DISTANCES_KIND_HETEROGENEOUS_TYPES) {
+	fprintf(output, "Relative %s matrix (name %s kind %lu) between %u heterogeneous objects by %s indexes:\n",
+		kindmeans, hwloc_distances_get_name(topology, dist[j]), dist[j]->kind,
+		dist[j]->nbobjs,
+		index_type != LSTOPO_INDEX_TYPE_PHYSICAL ? "logical" : "physical");
+      } else {
+	fprintf(output, "Relative %s matrix (name %s kind %lu) between %u %ss (depth %d) by %s indexes:\n",
+		kindmeans, hwloc_distances_get_name(topology, dist[j]), dist[j]->kind,
+		dist[j]->nbobjs,
+		hwloc_obj_type_string(dist[j]->objs[0]->type),
+		dist[j]->objs[0]->depth,
+		index_type != LSTOPO_INDEX_TYPE_PHYSICAL ? "logical" : "physical");
+      }
+      hwloc_utils_print_distance_matrix(output, dist[j]->nbobjs, dist[j]->objs, dist[j]->values, index_type != LSTOPO_INDEX_TYPE_PHYSICAL, dist[j]->kind & HWLOC_DISTANCES_KIND_HETEROGENEOUS_TYPES);
       hwloc_distances_release(topology, dist[j]);
     }
   }
